@@ -12,13 +12,23 @@ class DVCCA(_DCCA_base):
     """
     A class used to fit a DVCCA model.
 
+    :Citation:
+
+    Wang, Weiran, et al. "Deep variational canonical correlation analysis." arXiv preprint arXiv:1610.03454 (2016).
+
     https: // arxiv.org / pdf / 1610.03454.pdf
-    With pieces borrowed from the variational autoencoder implementation @
-    # https: // github.com / pytorch / examples / blob / master / vae / main.py
+
+    https: // github.com / pytorch / examples / blob / master / vae / main.py
+
     """
 
-    def __init__(self, latent_dims: int, encoders=None,
-                 decoders=None, private_encoders: Iterable[BaseEncoder] = None):
+    def __init__(
+        self,
+        latent_dims: int,
+        encoders=None,
+        decoders=None,
+        private_encoders: Iterable[BaseEncoder] = None,
+    ):
         """
         :param latent_dims: # latent dimensions
         :param encoders: list of encoder networks
@@ -44,7 +54,7 @@ class DVCCA(_DCCA_base):
         :return:
         """
         # Used when we get reconstructions
-        mu, logvar = self.encode(*args)
+        mu, logvar = self._encode(*args)
         if mle:
             z = mu
         else:
@@ -54,16 +64,16 @@ class DVCCA(_DCCA_base):
         if len(self.encoders) == 1:
             z = z * len(args)
         if self.private_encoders:
-            mu_p, logvar_p = self.encode_private(*args)
+            mu_p, logvar_p = self._encode_private(*args)
             if mle:
                 z_p = mu_p
             else:
                 z_dist = dist.Normal(mu_p, torch.exp(0.5 * logvar_p))
-                z = z_dist.rsample()
-            z = [torch.cat([z_] + z_p, dim=-1) for z_ in z]
+                z_p = z_dist.rsample()
+            z = [torch.cat((z_, z_p_), dim=-1) for z_, z_p_ in zip(z, z_p)]
         return z
 
-    def encode(self, *args):
+    def _encode(self, *args):
         """
         :param args:
         :return:
@@ -76,7 +86,7 @@ class DVCCA(_DCCA_base):
             logvar.append(logvar_i)
         return mu, logvar
 
-    def encode_private(self, *args):
+    def _encode_private(self, *args):
         """
         :param args:
         :return:
@@ -89,16 +99,16 @@ class DVCCA(_DCCA_base):
             logvar.append(logvar_i)
         return mu, logvar
 
-    def decode(self, z):
+    def _decode(self, z):
         """
         :param z:
         :return:
         """
         x = []
         for i, decoder in enumerate(self.decoders):
-            x_i = decoder(z)
+            x_i = F.sigmoid(decoder(z))
             x.append(x_i)
-        return tuple(x)
+        return x
 
     def recon(self, *args):
         """
@@ -106,22 +116,27 @@ class DVCCA(_DCCA_base):
         :return:
         """
         z = self(*args)
-        return [self.decode(z_i) for z_i in z][0]
+        return [self._decode(z_) for z_ in z]
 
     def loss(self, *args):
         """
         :param args:
         :return:
         """
-        mus, logvars = self.encode(*args)
+        mus, logvars = self._encode(*args)
         if self.private_encoders:
-            mus_p, logvars_p = self.encode_private(*args)
-            losses = [self.vcca_private_loss(*args, mu=mu, logvar=logvar, mu_p=mu_p, logvar_p=logvar_p) for
-                      (mu, logvar, mu_p, logvar_p) in
-                      zip(mus, logvars, mus_p, logvars_p)]
+            mus_p, logvars_p = self._encode_private(*args)
+            losses = [
+                self.vcca_private_loss(
+                    *args, mu=mu, logvar=logvar, mu_p=mu_p, logvar_p=logvar_p
+                )
+                for (mu, logvar, mu_p, logvar_p) in zip(mus, logvars, mus_p, logvars_p)
+            ]
         else:
-            losses = [self.vcca_loss(*args, mu=mu, logvar=logvar) for (mu, logvar) in
-                      zip(mus, logvars)]
+            losses = [
+                self.vcca_loss(*args, mu=mu, logvar=logvar)
+                for (mu, logvar) in zip(mus, logvars)
+            ]
         return torch.stack(losses).mean()
 
     def vcca_loss(self, *args, mu, logvar):
@@ -134,11 +149,16 @@ class DVCCA(_DCCA_base):
         batch_n = mu.shape[0]
         z_dist = dist.Normal(mu, torch.exp(0.5 * logvar))
         z = z_dist.rsample()
-        kl = torch.mean(-0.5 * torch.sum(1 + logvar - logvar.exp() - mu.pow(2), dim=1), dim=0)
-        recons = self.decode(z)
+        kl = torch.mean(
+            -0.5 * torch.sum(1 + logvar - logvar.exp() - mu.pow(2), dim=1), dim=0
+        )
+        recons = self._decode(z)
         bces = torch.stack(
-            [F.binary_cross_entropy(recon, arg, reduction='sum') / batch_n for recon, arg in
-             zip(recons, args)]).sum()
+            [
+                F.binary_cross_entropy(recon, arg, reduction="mean")
+                for recon, arg in zip(recons, args)
+            ]
+        ).sum()
         return kl + bces
 
     def vcca_private_loss(self, *args, mu, logvar, mu_p, logvar_p):
@@ -154,12 +174,24 @@ class DVCCA(_DCCA_base):
         z_p_dist = dist.Normal(mu_p, torch.exp(0.5 * logvar_p))
         z_p = z_p_dist.rsample()
         kl_p = torch.stack(
-            [torch.mean(-0.5 * torch.sum(1 + logvar_p - logvar_p.exp() - mu_p.pow(2), dim=1), dim=0) for
-             i, _ in enumerate(self.private_encoders)]).sum()
-        kl = torch.mean(-0.5 * torch.sum(1 + logvar - logvar.exp() - mu.pow(2), dim=1), dim=0)
+            [
+                torch.mean(
+                    -0.5
+                    * torch.sum(1 + logvar_p - logvar_p.exp() - mu_p.pow(2), dim=1),
+                    dim=0,
+                )
+                for i, _ in enumerate(self.private_encoders)
+            ]
+        ).sum()
+        kl = torch.mean(
+            -0.5 * torch.sum(1 + logvar - logvar.exp() - mu.pow(2), dim=1), dim=0
+        )
         z_combined = torch.cat([z, z_p], dim=-1)
-        recon = self.decode(z_combined)
+        recon = self._decode(z_combined)
         bces = torch.stack(
-            [F.binary_cross_entropy(recon[i], args[i], reduction='sum') / batch_n for i, _ in
-             enumerate(self.decoders)]).sum()
+            [
+                F.binary_cross_entropy(recon[i], args[i], reduction="sum") / batch_n
+                for i, _ in enumerate(self.decoders)
+            ]
+        ).sum()
         return kl + kl_p + bces
